// LibAvWrap -- .NET wrapper for the libav* libraries
// --------------------------------------------------
// By Christian Richardt <christian@richardt.name>
// (based on AvDn 0.7, www.informikon.com/tutorials/)
//
// $Id: Packet.h 436 2010-10-15 16:26:38Z cr333 $

#pragma once

using namespace System;
using namespace System::Text;
using namespace System::Runtime::InteropServices;

extern "C" {
#include "libavformat/avformat.h"
}

#include "Wrapper.h"

///<summary>Wrapper for some lavc and lavf structures.</summary>
namespace LibAvWrap
{
	///<summary>Wrapper for the AVPacket structure.</summary>
	public ref class LibAvPacket : Wrapper<AVPacket>
	{
// constructors and destructor --------------------------------------------------------------------

	public:
		///<summary>Allocates a new packet.</summary>
		LibAvPacket() : Wrapper<AVPacket>()
		{
			pointer = new AVPacket;
			pointer->size = 0;
			av_init_packet(pointer);
		}

		///<summary>Frees and destructs a packet.</summary>
		~LibAvPacket()
		{
			if(pointer != NULL)
			{
				av_free_packet(pointer);
				delete pointer;
				pointer = NULL;
			}
		}

	protected:
		///<summary>Finaliser, called by garbage collector.</summary>
		!LibAvPacket()
		{
			if(pointer != NULL)
			{
				av_free_packet(pointer);
				delete pointer;
				pointer = NULL;
			}
		}

// member functions -------------------------------------------------------------------------------
	public:
		///<summary>Get packet data pointer.</summary>
		uint8_t* GetData() { return (uint8_t*)pointer->data; }
		
		///<summary>Set packet data pointer.</summary>
		void SetData(IntPtr d) { pointer->data = (uint8_t*)(void*)d; }

		///<summary>Free this packet.</summary>
		void Free() { av_free_packet(pointer); }

// properties -------------------------------------------------------------------------------------

		///<summary>Packet size.</summary>
		property int Size {
			int get() { return pointer->size; }
			void set(int s) { pointer->size = s; }
		}

		///<summary>Stream index.</summary>
		property int StreamIndex {
			int get() { return pointer->stream_index; }
			void set(int index) { pointer->stream_index = index; }
		}
	};

} // namespace LibAvWrap