/*
This source code is part of the Floating Textures test program, which
is a derivative of the complete Floating Textures program containing
the offline version with prereconstructed geometry and prerecorded
video. 
For more information on Floating Textures please have a look at the article

"Floating Textures" 
by M. Eisemann, B. De Decker, M. Magnor, 
P. Bekaert, E. de Aguiar, N. Ahmed, C. Theobalt, A. Sellent
Computer Graphics Forum (Proc. Eurographics EG'08), vol. 27, no. 2, April 2008 


Copyright (C) 2008 Martin Eisemann

This program is free software; you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by the Free Software 
Foundation; either version 3 of the License, or (at your option) any later 
version.

This program is distributed in the hope that it will be useful, but WITHOUT 
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, see <http://www.gnu.org/licenses/>.
*/

/**
 * \file Warping.frag
 *
 * \brief Updates the given Optical Flow Field using the algorithm by Brox et al. "High Accuracy Optical Flow Estimation Based on a Theory for Warping"
 *
 * \author Martin Eisemann (ME)
 *
 */

// enable draw_buffers //
#extension GL_ARB_draw_buffers : enable

uniform vec2 dxy;  // pixelsize in texture coordinates

uniform sampler2D texSource;
uniform sampler2D texDX;
uniform sampler2D texDY;
uniform sampler2D texDXX;
uniform sampler2D texDXY;
uniform sampler2D texDYY;
uniform sampler2D texUVField;

void main()
{
  // I1 -> I2
  vec2 texCoord  = gl_TexCoord[0].xy;
  vec4 temp = vec4(texCoord,texCoord) - vec4(dxy,dxy) * texture2D(texUVField, texCoord);
  vec2 wTexCoord = temp.xy;
  
  float IW   = texture2D(texSource, wTexCoord).r;
  float IxW  = texture2D(texDX, wTexCoord).r;
  float IyW  = texture2D(texDY, wTexCoord).r;
  float IxxW = texture2D(texDXX,wTexCoord).r;
  float IxyW = texture2D(texDXY,wTexCoord).r;
  float IyyW = texture2D(texDYY,wTexCoord).r;
  
  float I2   = texture2D(texSource, texCoord).g;
  float I2x  = texture2D(texDX, texCoord).g;
  float I2y  = texture2D(texDY, texCoord).g;

  float Iz   = IW  - I2;
  float Ixz = IxW - I2x;
  float Iyz = IyW - I2y;  
  
  // I2 -> I1  
  wTexCoord = temp.zw;

  float IW2   = texture2D(texSource, wTexCoord).g;
  float IxW2  = texture2D(texDX, wTexCoord).g;
  float IyW2  = texture2D(texDY, wTexCoord).g;
  float IxxW2 = texture2D(texDXX,wTexCoord).g;
  float IxyW2 = texture2D(texDXY,wTexCoord).g;
  float IyyW2 = texture2D(texDYY,wTexCoord).g;

  float I22   = texture2D(texSource, texCoord).r;
  float I2x2  = texture2D(texDX, texCoord).r;
  float I2y2  = texture2D(texDY, texCoord).r;

  float Iz2   = IW2  - I22;
  float Ixz2 = IxW2 - I2x2;
  float Iyz2 = IyW2 - I2y2;


  gl_FragData[0] = vec4(IxW, IyW, IxW2, IyW2);
  gl_FragData[1] = vec4(Iz, IxxW, Iz2, IxxW2);
  gl_FragData[2] = vec4(IxyW, Ixz, IxyW2, Ixz2);
  gl_FragData[3] = vec4(IyyW, Iyz, IyyW2, Iyz2);
} 
