/*
This source code is part of the Floating Textures test program, which
is a derivative of the complete Floating Textures program containing
the offline version with prereconstructed geometry and prerecorded
video. 
For more information on Floating Textures please have a look at the article

"Floating Textures" 
by M. Eisemann, B. De Decker, M. Magnor, 
P. Bekaert, E. de Aguiar, N. Ahmed, C. Theobalt, A. Sellent
Computer Graphics Forum (Proc. Eurographics EG'08), vol. 27, no. 2, April 2008 


Copyright (C) 2008 Martin Eisemann

This program is free software; you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by the Free Software 
Foundation; either version 3 of the License, or (at your option) any later 
version.

This program is distributed in the hope that it will be useful, but WITHOUT 
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS 
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with 
this program; if not, see <http://www.gnu.org/licenses/>.
*/

/**
 * \file morph.frag
 *
 * \brief Fragment shader for warping two input textures, defined by a flow field and blending them together
 *
 */ 

uniform sampler2D source;
uniform sampler2D target;

uniform sampler2D opticalflow;

// This is the time value [0..1]. 0: source, 1: target
uniform vec4 value; // value.x is value.y, value.z is 1.0-value.x, value.w is value.z

uniform vec4 dxy; // step size, depending on texture resolution NxM -> (1/N,1/M)
                  // dxy.x and dxy.z -> x-direction
                  // dxy.y and dxy.w -> y-direction


void main() {

  // original coordinates //
  vec4 tC = vec4(gl_TexCoord[0].xy, gl_TexCoord[0].xy);

  // calculate the warped coordinates, depending on the flow field //
  vec4 flows = texture2D(opticalflow, gl_TexCoord[0].xy); // xy is the warp from S to T, zw from T to S
  vec4 warpTC = tC - value * dxy * flows;

  vec4 colorSource = texture2D(source, warpTC.xy);
  vec4 colorTarget = texture2D(target, warpTC.zw);

  gl_FragColor = mix(colorSource, colorTarget, value.xyxy);  
}

