// $Id: SwissRanger4000Source.h 262 2010-07-28 13:25:26Z cr333 $
#pragma once

#include <windows.h>
#include "libMesaSR/libMesaSR.h"
#include "Enums.h"
#include "ImageEntry.h"

using namespace System;
using namespace System::Collections::Generic;
using namespace VideoLib;
using namespace VideoLib::Frames;

namespace VideoLib { namespace SR4000 {

	public ref class SwissRanger4000Source : Node
	{
	public:
		SwissRanger4000Source();
		~SwissRanger4000Source();

	protected:
		!SwissRanger4000Source();
		typedef VideoLib::SR4000::AcquireMode AM;

// Node interface ---------------------------------------------------------------------------------
	public:		
		virtual void Init() override;
		virtual void Process() override;
		virtual Void UpdateStatusImpl() override;

// wrapped libMesaSR functions --------------------------------------------------------------------
	protected:
		void Acquire();
		void Close();
		IEnumerable<ImageEntry>^ GetImageList();
		void OpenDialog(int mode, HWND parent);
		void OpenUSB(unsigned int serialNumber);
		void OpenUSB() { OpenUSB(0); }

	private:
		SRCAM* _srcam;

// additional functions ---------------------------------------------------------------------------
	protected:
		void ApplySettings(bool force);
		void ApplySettings() { ApplySettings(false); }

	public:
		void EnableMode( AM acquireMode);
		void DisableMode(AM acquireMode);
		unsigned char TimeFromMs(float milliseconds);
		float TimeToMs(unsigned char time);

// properties -------------------------------------------------------------------------------------
	private:
		int   _sequenceNumber;

		bool  _dirty; // dirty flag which forces all settings to be re-applied
		AM    _acquireMode;
		float _integrationTime;
		int   _timeout; 
		bool  _autoExposure;

	public:
		property int Width  { int get(); }
		property int Height { int get(); }

		/// <summary>The sequence number of the most recently decoded frame.</summary>
		property int   SequenceNumber  { int   get() { return _sequenceNumber;  } protected: void set(int value) { _sequenceNumber = value; } }
		property AM    AcquireMode     { AM    get() { return _acquireMode;     } void set(AM    value) { _acquireMode     = value; _dirty = true; } }
		property float IntegrationTime { float get() { return _integrationTime; } void set(float value) { _integrationTime = value; _dirty = true; } }
		property int   Timeout         { int   get() { return _timeout;         } void set(int   value) { _timeout         = value; _dirty = true; } }
		property bool  AutoExposure    { bool  get() { return _autoExposure;    } void set(bool  value) { _autoExposure    = value; _dirty = true; } }

		property float AEMinIntegrationTime;
		property float AEMaxIntegrationTime;
		property unsigned char AEPercentOverPos;
		property unsigned char AEDesiredPos;

		property bool ConfidenceMap { bool get(); void set(bool value); }
		property bool Coordinates;
		property bool Triggered     { bool get(); void set(bool value); }
	};

}} // closing namespace VideoLib::SR4000