﻿// $Id: VideoFrame.cs 413 2010-10-04 19:42:52Z cr333 $
using System;

namespace VideoLib.Frames
{
	/// <summary>Implements a generic video frame.</summary>
	public abstract class VideoFrame : Frame
	{
		/// <summary>Gets the width of the video frame.</summary>
		public int Width  { get; protected set; }

		/// <summary>Gets the height of the video frame.</summary>
		public int Height { get; protected set; }

		/// <summary>Gets the format of the pixels.</summary>
		public PixelFormat Format { get; protected set; }

		/// <summary>Initialises a new instance of the <see cref="VideoFrame"/> class.</summary>
		public VideoFrame() : base() { }

		/// <summary>Initialises a new instance of the <see cref="VideoFrame"/> class.</summary>
		/// <param name="width">The width of the video frame.</param>
		/// <param name="height">The height of the video frame.</param>
		/// <param name="format">The pixel format of the video frame.</param>
		public VideoFrame(int width, int height, PixelFormat format) : base()
		{
			// sanity checks
			if (width  <= 0) throw new ArgumentOutOfRangeException("width",  "Frame width must be positive.");
			if (height <= 0) throw new ArgumentOutOfRangeException("height", "Frame height must be positive.");

			Width  = width;
			Height = height;
			Format = format;
		}

		/// <summary>Returns a <see cref="System.String"/> that represents this instance.</summary>
		public override string ToString()
		{
			return String.Format("{0} ({1}x{2}, {3})", this.GetType().Name, Width, Height, Format);
		}
	}
}
